#!/usr/bin/env python
# coding=utf-8

"""
demodocus is Python CLI program to quickly check synchronization maps
"""

from __future__ import absolute_import
import argparse
import sys

from demodocus.__main__ import main

__author__ = "Alberto Pettarin"
__copyright__ = "Copyright 2015, Alberto Pettarin (www.albertopettarin.it)"
__license__ = "MIT"
__version__ = "1.0.0"
__email__ = "alberto@albertopettarin.it"
__status__ = "Production"



